{% load i18n %}
<div class="paginator">
 {% if show_first %}<a href="?{%if extra_query%}{{extra_query}}&{%endif%}page=1" title="{% trans "First Page" %}">&laquo;</a>{% endif %}
 {% if has_previous %}<a href="?{%if extra_query%}{{extra_query}}&{%endif%}page={{previous}}" title="{% trans "Previous Page" %}">&lt;</a></span>{% endif %}
{% for pagenum in page_numbers %}
{%  ifequal pagenum page %}
 <span class="current-page">{{pagenum}}</span>
{%  else %}
 <a href="?{%if extra_query%}{{extra_query}}&{%endif%}page={{pagenum}}" title="{% blocktrans %}Page {{pagenum}}<{% endblocktrans %}">{{pagenum}}</a>
{%  endifequal %}
{% endfor %}
{% if has_next %}<a href="?{%if extra_query%}{{extra_query}}&{%endif%}page={{next}}" title="{% trans "Next Page" %}">&gt;</a></span>{% endif %}
{% if show_last %}<a href="?{%if extra_query%}{{extra_query}}&{%endif%}page={{pages}}" title="{% trans "Last Page" %}">&raquo;</a></span>{% endif %}
 <span class="page-count">{{pages}} pages</span>
</div>
